package com.xkrs.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.dao.SettingDao;
import com.xkrs.model.entity.SettingEntity;
import com.xkrs.model.qo.SettingQo;
import com.xkrs.service.SettingService;
import org.apache.http.util.TextUtils;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

@Service
public class SettingServiceImpl implements SettingService {

    @Resource
    private SettingDao settingDao;

    /**
     * 更新审核开关状态
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public String updateSwitchState(SettingQo settingQo) {
        Locale locale = LocaleContextHolder.getLocale();
        if (TextUtils.isEmpty(settingQo.getSwitchState())) {
            return outputEncapsulationObject(PromptMessageEnum.PARAM_NULL, "switchState参数不能为空", locale);
        }
        List<SettingEntity> settingEntityList = settingDao.findAll();
        if (settingEntityList.isEmpty()) {
            SettingEntity settingEntity = new SettingEntity();
            settingEntity.setSwitchState(settingQo.getSwitchState());
            settingDao.save(settingEntity);
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "修改成功", locale);
        }
        for (SettingEntity settingEntity : settingEntityList) {
            settingDao.updateSwitchStateById(settingEntity.getId(), settingQo.getSwitchState());
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "修改成功", locale);
    }

    /**
     * 查询审核开关状态
     */
    @Override
    public String querySwitchState() {
        Locale locale = LocaleContextHolder.getLocale();
        List<SettingEntity> settingEntityList = settingDao.findAll();
        if (settingEntityList.isEmpty()) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "暂时还没有状态信息", locale);
        }
        SettingEntity settingEntity = settingEntityList.get(0);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, settingEntity.getSwitchState(), locale);
    }
}
